from cubicweb.devtools import buildconfig
from cubicweb.devtools.testlib import RealDBTest

def setup_module(options):
    if options.dbname is None:
        raise Exception('the <dbname> options is required')
    RealDatabaseTC.configcls = buildconfig(options.dbuser, options.dbpassword,
                                           options.dbname, options.euser,
                                           options.epassword)

class RealDatabaseTC(RealDBTest):
    configcls = None # set by setup_module()

    def test_all_primaries(self):
        for rset in self.iter_individual_rsets(limit=50):
            yield self.view, 'primary', rset, rset.req.reset_headers()
            yield self.view, 'oneline', rset, rset.req.reset_headers()
    
    def test_startup_views(self):
        for vid in self.list_startup_views():
            req = self.request()
            yield self.view, vid, None, req


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



"""drh web ui"""

from cubicweb.web import uicfg

uicfg.actionbox_appearsin_addmenu.tag_subject_of(('Person', 'concerned_by', '*'), True)
uicfg.actionbox_appearsin_addmenu.tag_object_of(('*', 'todo_by', 'Person'), True)
uicfg.actionbox_appearsin_addmenu.tag_object_of(('Application', 'for_person', 'Person'), True)
uicfg.actionbox_appearsin_addmenu.tag_subject_of(('School', 'filed_under', 'Folder'), False)
uicfg.actionbox_appearsin_addmenu.tag_object_of(('Application', 'for_person', 'Person'), True)
uicfg.autoform_section.tag_subject_of(('School', 'use_email', '*'), 'main', 'attributes')
uicfg.autoform_section.tag_subject_of(('School', 'phone', '*'), 'main', 'attributes')
uicfg.primaryview_section.tag_subject_of(('Person', 'concerned_by', '*'), 'hidden')
uicfg.primaryview_section.tag_object_of(('*', 'topic', 'Application'), 'hidden')
uicfg.autoform_section.tag_subject_of(('Application', 'for_person', 'Person'), 'main', 'attributes')



# -*- coding: utf-8 -*-

#delete after the person's cube migration
add_entity_type('PostalAddress')

if confirm('backport person address?'):
    for person_eid, address, description in rql(
        'Any X,XA,XD WHERE X is Person, NOT X address NULL, X address XA, X description XD'):
        ## move address to description if it's longer than 256 character
        if len(address) < 256:
            rql('INSERT PostalAddress P: '
                'P street %(street)s , P city %(city)s, P postalcode %(pc)s, D postal_address P '
                'WHERE D is Person, D eid %(eid)s',
                {'street':address, 'city':u'???', 'pc':u'???', 'eid': person_eid},
                ask_confirm=False)
        else:
            if description:
                description = description + ' ' + address
            else:
                description = address
            rql("SET X description %(D)s WHERE X is Person, X eid %(eid)s",
                {'D': description, 'eid':person_eid})
    checkpoint()

add_entity_type('Application')

if confirm('add application wf?'):
    ##add state and transition on Apllication and remove it from Person
    appl  = add_state(_('application'),                'Application', initial=True)
    prop  = add_state(_('send interview proposition'), 'Application')
    itne  = add_state(_('interview time negociation'), 'Application')
    itpl  = add_state(_('interview planned'),          'Application')
    itju  = add_state(_('interview judgment'),         'Application')
    spoa  = add_state(_('send positive answer'),       'Application')
    snea  = add_state(_('send negative answer'),       'Application')
    refu  = add_state(_('refused'),                    'Application')
    recr  = add_state(_('recruited'),                  'Application')
    canc  = add_state(_('canceled'),                   'Application')
    add_transition(_('cancel application'), 'Application', (appl, prop, itne, itpl, itju, spoa, snea, refu, recr),  canc)
    add_transition(_('discard application'), 'Application', (appl, itju),  snea)
    add_transition(_('negative answer sent'), 'Application', (snea,),  refu)
    add_transition(_('propose interview'), 'Application', (appl,),  prop)
    add_transition(_('schedule interview'), 'Application', (appl,),  itne)
    add_transition(_('interview scheduled'), 'Application', (itne,),  itpl)
    add_transition(_('interview done'), 'Application', (itpl,),  itju)
    add_transition(_('propose new interview'), 'Application', (itju,),  prop)
    add_transition(_('accept application'), 'Application', (itju, ),  spoa)
    add_transition(_('application accepted'), 'Application', (spoa,),  recr)
    checkpoint()

if confirm('insert application for each person?'):

    state_changed ={u'jugement candidature': u'application',
                    u'envoi réponse négative': u'send negative answer',
                    u'envoi proposition entretien': u'send interview proposition',
                    u'négociation horaire entretien': u'interview time negociation',
                    u'jugement entretien': u'interview judgment',
                    u'entretien convenu': u'send interview proposition',
                    u'envoi réponse positive': u'send positive answer',
                    u'refusé': u'refused',
                    u'recruté': u'recruited',
                    u'abandon': u'canceled',
                    u'annulé': u'canceled'}

    ##migrate state form person to application

    for person in rql('Any P,S,SN WHERE P is Person, P in_state S, S name SN').entities():
        assert rql(u'INSERT Application X : X for_person P, X in_state S '
                   'WHERE P eid %(eid)s, S name %(state)s',
                   {'eid': person.eid, 'state': state_changed[person.state]},
                   ask_confirm=False)
    checkpoint()

# delete state for Person
drop_relation_definition('Person', 'in_state', 'State')
drop_relation_definition('Person', 'wf_info_for', 'TrInfo')
drop_attribute('Person', 'address')

rql('DELETE State S WHERE S state_of X, X name "Person"')




# application are taggable
add_relation_definition('Tag','tags','Application')
# move tags from person to applications
rql('SET T tags A WHERE T tags P, P is Person, A for_person P')
rql('DELETE T tags P WHERE P is Person')
checkpoint()

# fix wflow
pending = add_state(u'recruitment pending','Application')
pos = rql('Any S WHERE S name "send positive answer"')[0][0]
add_transition(u'sent positive answer', 'Application', (pos,), pending)

rql('SET S name "received" WHERE S is State, S name "application"', ask_confirm=False)
rql('SET S name "rejected" WHERE S is State, S name "refused"', ask_confirm=False)
rql('SET S name "interview scheduling" WHERE S is State, S name "interview time negociation"', ask_confirm=False)
rql('SET S name "notify accept" WHERE S is State, S name "send positive answer"', ask_confirm=False)
rql('SET S name "notify reject" WHERE S is State, S name "send negative answer"', ask_confirm=False)
rql('SET S name "decision pending" WHERE S is State, S name "interview judgment"', ask_confirm=False)

rql('SET T name "reject application" WHERE T is Transition, T name "discard application"', ask_confirm=False)
rql('SET T name "recruitment accepted" WHERE T is Transition, T name "application accepted"', ask_confirm=False)
rql('SET T name "accept interview" WHERE T is Transition, T name "interview scheduled"', ask_confirm=False)
rql('SET T name "send negative answer" WHERE T is Transition, T name "negative answer sent"', ask_confirm=False)
rql('SET T name "send positive answer" WHERE T is Transition, T name "sent positive answer"', ask_confirm=False)

rql('SET A in_state S WHERE S name "interview scheduling", A in_state X, X name "send interview proposition"', ask_confirm=False)

rql('DELETE Transition T WHERE T name "propose interview"', ask_confirm=False)
rql('DELETE Transition T WHERE T name "propose new interview"', ask_confirm=False)
rql('DELETE State S WHERE S name "send interview proposition"', ask_confirm=False)

rql('SET S allowed_transition T WHERE S name "recruitment pending", T name "cancel application"', ask_confirm=False)
rql('SET S allowed_transition T WHERE S name "recruitment pending", T name "recruitment accepted"', ask_confirm=False)
rql('SET S allowed_transition T WHERE S name "decision pending", T name "schedule interview"', ask_confirm=False)

rql('DELETE S allowed_transition T WHERE S name "recruited", T name "cancel application"', ask_confirm=False)
rql('DELETE S allowed_transition T WHERE S name "rejected", T name "cancel application"', ask_confirm=False)
rql('DELETE S allowed_transition T WHERE S name "notify accept", T name "recruitment accepted"', ask_confirm=False)
checkpoint()



from logilab.mtconverter import xml_escape

from cubicweb.web.views import startup

class IndexView(startup.ManageView):
    __regid__ = 'index'
    title = _('Index')

    def call(self):
        _ = self._cw._
        user = self._cw.user
        self.w(u'<h1>%s</h1>' % self._cw.property_value('ui.site-title'))
        # email addresses not linked
        rql = 'Any X WHERE NOT P use_email X'
        title = u'email addresses not linked to a person'
        rset = self._cw.execute(rql)
        if rset:
            self.w(u'<p><a href="%s">%s %s</a></p>'
                   % (xml_escape(self.build_url(rql=rql, vtitle=title)),
                      len(rset), title))
        # email threads not linked to an application
        rql = 'Any T WHERE T is EmailThread, NOT T topic X'
        title = u'message threads without topic'
        rset = self._cw.execute(rql)
        if rset:
            self.w(u'<p><a href="%s">%s %s</a></p>'
                   % (xml_escape(self.build_url(rql=rql, vtitle=title)),
                      len(rset), title))
        # candidatures en attente
        rset = self._cw.execute('Any A,P,group_concat(TN),E,B '
                                'GROUPBY A,P,E,B,CD ORDERBY CD '
                                'WHERE A is Application, A in_state X, '
                                'X name "received", '
                                'A for_person P, P has_studied_in E?, '
                                'P birthday B, T? tags A, T name TN, '
                                'A creation_date CD')
        if rset:
            self.w(u'<h2>%s</h2>' % _('Juger candidatures'))
            self.wview('table',rset,'null')
        else:
            self.w(u'<p>%s</p>' % _('aucune candidature en attente'))




def registration_callback(vreg):
    vreg.register(IndexView, clear=True)



from cubicweb.selectors import implements, rql_condition
from cubicweb.web.component import RelatedObjectsVComponent

class SentMailVComponent(RelatedObjectsVComponent):
    """email sent by this person"""
    __regid__ = 'sentmail'
    __select__ = RelatedObjectsVComponent.__select__ & implements('Person') & rql_condition('X use_email EA, E sender EA')
    rtype = 'use_email'
    role = 'subject'
    order = 40
    # reuse generated message id
    title = _('contentnavigation_sentmail')

    def rql(self):
        """override this method if you want to use a custom rql query"""
        return 'Any E ORDERBY D DESC WHERE P use_email EA, E sender EA, E date D, P eid %(x)s'

class ThreadTopicVComponent(RelatedObjectsVComponent):
    """email in threads related to this topic"""
    __regid__ = 'threadtopic'
    __select__ = RelatedObjectsVComponent.__select__ & implements('Application') & rql_condition('E in_thread T, T topic X')
    rtype = 'topic'
    role = 'object'
    order = 40
    # reuse generated message id
    title = _('contentnavigation_mailtopic')

    def rql(self):
        """override this method if you want to use a custom rql query"""
        return 'Any E ORDERBY D DESC WHERE E date D, E in_thread T, T topic A, A eid %(x)s'




from yams.buildobjs import (EntityType, RelationDefinition, SubjectRelation,
                            String, Date, Datetime)
from cubicweb.schema import WorkflowableEntityType


class School(EntityType):
    """an (high) school"""
    name        = String(required=True, fulltextindexed=True, maxsize=128)
    address     = String(maxsize=512)
    description = String(fulltextindexed=True)

    phone     = SubjectRelation('PhoneNumber', composite='subject')
    use_email = SubjectRelation('EmailAddress', composite='subject')


class Application(WorkflowableEntityType):
    date = Datetime(default='TODAY', required=True)
    for_person = SubjectRelation('Person', cardinality='1*', composite='object')


class comments(RelationDefinition):
    subject = 'Comment'
    object = 'Person', 'Task', 'Event'
    cardinality = '1*'
    composite = 'object'

class tags(RelationDefinition):
    subject = 'Tag'
    object = ('Person', 'Application')


class birthday(RelationDefinition):
    subject = 'Person'
    object = 'Date'

class concerned_by(RelationDefinition):
    subject = 'Person'
    object = 'File'

class has_studied_in(RelationDefinition):
    """used to indicate an estabishment where a person has been studying"""
    # XXX promotion?
    subject = 'Person'
    object = 'School'


class interested_in(RelationDefinition):
    subject = ('Person', 'CWUser')
    object = 'Event'


class todo_by(RelationDefinition):
    subject = 'Task'
    object = 'Person'


class topic(RelationDefinition):
    subject = 'EmailThread'
    object = 'Application'




# add topic relation
add_relation_definition('EmailThread','topic','Application')
sync_schema_props_perms()

# set topics
rql('SET T topic A WHERE E in_thread T, E sender EA, P primary_email EA, A for_person P')
rql('SET T topic A WHERE E in_thread T, E recipients EA, P primary_email EA, A for_person P, NOT T topic A')



from cubicweb.interfaces import ITree
from cubicweb.mixins import TreeMixIn

from cubicweb.entities import AnyEntity, fetch_config

from cubes.email.entities import EmailThread as EmailEmailThread

class School(AnyEntity):
    __regid__ = 'School'
    fetch_attrs, fetch_order = fetch_config(['name'])


class EmailThread(TreeMixIn, EmailEmailThread):

    __implements__ = EmailEmailThread.__implements__ + (ITree,)

    tree_attribute = 'topic'
    parent_target = 'object'
    children_target = 'subject'

    def parent(self):
        """for breadcrumbs"""
        return self.topic and self.topic[0] or None


class Application(TreeMixIn, AnyEntity):

    __regid__ = 'Application'
    __implements__ = AnyEntity.__implements__ + (ITree,)

    tree_attribute = 'for_person'
    parent_target = 'object'
    children_target = 'subject'

    def parent(self):
        """for breadcrumbs"""
        return self.for_person and self.for_person[0] or None




# -*- coding: utf-8 -*-

from cubicweb.web.box import BoxTemplate
from cubicweb.web.htmlwidgets import BoxWidget, BoxLink

from logilab.mtconverter import xml_escape
from cubicweb.web.box import EntityBoxTemplate

from cubicweb.selectors import implements, rql_condition
from cubicweb.web.htmlwidgets import SideBoxWidget, BoxLink

class StartupViewsBox(BoxTemplate):
    """display a box containing links to all startup views"""
    __regid__ = 'drh_workflow_box'
    visible = True # disabled by default
    title = _('State')
    order = 70

    def call(self, **kwargs):
        box = BoxWidget(self._cw._(self.title), self.__regid__)
        rset = self._cw.execute('Any S,SN,count(A) GROUPBY S,SN ORDERBY SN '
                                'WHERE A is Application, A in_state S, S name SN')
        for eid, state, count in rset:
            rql_syn = ('Any A,P,group_concat(TN),E,B '
                       'GROUPBY A,P,E,B,CD ORDERBY CD '
                       'WHERE A in_state X, X eid %s, '
                       'A for_person P, P is Person, '
                       'T? tags A, T name TN, P has_studied_in E?, '
                       'P birthday B?, A creation_date CD')
            url = self._cw.build_url(rql=rql_syn % eid,
                                 vtitle=self._cw._(state))
            label = u'%s: %s' % (state, count)
            box.append(BoxLink(url, label))

        if not box.is_empty():
            box.render(self.w)


class AttachmentsDownloadBox(EntityBoxTemplate):
    """
    A box containing all downloadable attachments concerned by Person.
    """
    __regid__ = 'concerned_by_box'
    __select__ = EntityBoxTemplate.__select__ & implements('Person')
    rtype = 'concerned_by'
    target = 'subject'
    order = 0

    def cell_call(self, row, col, **kwargs):
        entity = self.cw_rset.get_entity(row, col)
        req = self._cw
        self.w(u'<div class="sideBox">')
        title = req._('concerned_by')
        self.w(u'<div class="sideBoxTitle downloadBoxTitle"><span>%s</span></div>'
            % xml_escape(title))
        self.w(u'<div class="sideBox downloadBox"><div class="sideBoxBody">')
        for attachment in entity.concerned_by:
            self.w(u'<div><a href="%s"><img src="%s" alt="%s"/> %s</a>'
                   % (xml_escape(attachment.download_url()),
                      req.external_resource('DOWNLOAD_ICON'),
                      _('download icon'), xml_escape(attachment.dc_title())))
            self.w(u'</div>')
        self.w(u'</div>\n</div>\n</div>\n')


class PeopleBox(EntityBoxTemplate):
    __regid__ = '123people_box'
    __select__ = EntityBoxTemplate.__select__ & implements('Person')
    order = 25

    def cell_call(self, row, col, **kwargs):
        entity = self.cw_rset.get_entity(row, col)
        firstname = entity.firstname
        surname = entity.surname
        box = SideBoxWidget(self._cw._('The url\'s Person on 123people '),
                            'person_on_123people')
        box.append(BoxLink('http://www.123people.com/s/%s+%s/world' % (firstname, surname),
                           xml_escape('%s %s' % (firstname, surname))))
        self.w(box.render())



from os import makedirs
from os.path import join, exists

from cubicweb.server import hook
from cubicweb.server.sources import storages

class ServerStartupHook(hook.Hook):
    __regid__ = 'drh.serverstartup'
    events = ('server_startup', 'server_maintenance')

    def __call__(self):
        bfssdir = join(self.repo.config.appdatahome, 'bfss')
        if not exists(bfssdir):
            makedirs(bfssdir)
            print 'created', bfssdir
        storage = storages.BytesFileSystemStorage(bfssdir)
        storages.set_attribute_storage(self.repo, 'File', 'data', storage)



"""DRH TEST APPLICATION

Copyright (c) 2003-2004 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from cubicweb.devtools.testlib import AutomaticWebTest

if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



import os
import pwd


def getlogin():
    """avoid usinng os.getlogin() because of strange tty / stdin problems
    (man 3 getlogin)
    Another solution would be to use $LOGNAME, $USER or $USERNAME
    """
    return pwd.getpwuid(os.getuid())[0]


def update_parser(parser):
    login = getlogin()
    parser.add_option('-u', '--dbuser', dest='dbuser', action='store',
                      default=login, help="database user")
    parser.add_option('-w', '--dbpassword', dest='dbpassword', action='store',
                      default=login, help="database name")
    parser.add_option('-n', '--dbname', dest='dbname', action='store',
                      default=None, help="database name")
    parser.add_option('--euser', dest='euser', action='store',
                      default=login, help="esuer name")
    parser.add_option('--epassword', dest='epassword', action='store',
                      default=login, help="euser's password' name")
    return parser



rcvd = add_state(_('received'),             'Application', initial=True)
schd = add_state(_('interview scheduling'), 'Application')
plan = add_state(_('interview planned'),    'Application')
deci = add_state(_('decision pending'),     'Application')
nrej = add_state(_('notify reject'),        'Application')
nacp = add_state(_('notiy accept'),         'Application')
recp = add_state(_('recruitment pending'),  'Application')
recd = add_state(_('recruited'),            'Application')
rejt = add_state(_('rejected'),             'Application')
canc = add_state(_('canceled'),             'Application')

add_transition(_('cancel application'),  'Application', (rcvd, schd, plan, deci, nrej, nacp, recp),  canc)
add_transition(_('schedule interview'),  'Application', (rcvd, deci), schd)
add_transition(_('reject application'),  'Application', (rcvd, deci), nrej)
add_transition(_('accept application'),  'Application', (deci,), nacp)
add_transition(_('send negative answer'),'Application', (nrej,), rejt)
add_transition(_('accept interview'),    'Application', (schd,), plan)
add_transition(_('send positive answer'),'Application', (nacp,), recp)
add_transition(_('recruited accepted'),  'Application', (recp,), recd)
add_transition(_('interview done'),      'Application', (plan,), deci)

checkpoint()



# pylint: disable-msg=W0622
"""cubicweb-drh packaging information"""

modname = 'drh'
distname = "cubicweb-drh"

numversion = (0, 18, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'

author = "Logilab"
author_email = "contact@logilab.fr"

description = "recruitment application based on the CubicWeb framework"

web = 'http://www.cubicweb.org/project/%s' % distname

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

__depends__ = {'cubicweb': '>= 3.8.0',
               'cubicweb-file': None,
               'cubicweb-email': None,
               'cubicweb-person': None,
               'cubicweb-addressbook': None,
               'cubicweb-folder': None,
               'cubicweb-tag': None,
               'cubicweb-comment': None,
               # could be moved out
               'cubicweb-basket': None,
               'cubicweb-event': None,
               'cubicweb-task': None,
               }

# packaging ###

from os import listdir as _listdir
from os.path import join, isdir, exists, dirname
from glob import glob

THIS_CUBE_DIR = join('share', 'cubicweb', 'cubes', modname)

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')
            and not isdir(join(dirpath, fname))]

data_files = [
    # common files
    [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
    ]
# check for possible extended cube layout
for dirname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'i18n', 'migration'):
    if isdir(dirname):
        data_files.append([join(THIS_CUBE_DIR, dirname), listdir(dirname)])
# Note: here, you'll need to add subdirectories if you want
# them to be included in the debian package
